if select(6, GetAddOnInfo("CowTip_" .. (debugstack():match("[C%.][o%.][w%.]Tip\\Modules\\(.-)\\") or debugstack():match("[C%.][o%.][w%.]Tip\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 42315 $"):match("%d+"))

local CowTip = CowTip
local CowTip_Text = CowTip:NewModule("Text", "AceEvent-2.0")
local self = CowTip_Text
if CowTip.revision < VERSION then
	CowTip.version = "r" .. VERSION
	CowTip.revision = VERSION
	CowTip.date = ("$Date: 2007-06-30 05:05:30 -0400 (Sat, 30 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
CowTip.desc = "Allow for customized text on the tooltip"

local L = CowTip:L("CowTip_Text")

local DogTag = AceLibrary("DogTag-1.0")

local leftGameTooltipStrings = CowTip.leftGameTooltipStrings
local rightGameTooltipStrings = CowTip.rightGameTooltipStrings

local _G = _G
local IsShiftKeyDown = _G.IsShiftKeyDown
local ChatFontNormal = _G.ChatFontNormal
local CreateFrame = _G.CreateFrame
local UIParent = _G.UIParent
local GameTooltip = _G.GameTooltip
local ipairs = _G.ipairs

function CowTip_Text:OnInitialize()
	self.db = CowTip:AcquireDBNamespace("Text")
	CowTip:RegisterDefaults("Text", "profile", {
		lefts = {
			['*'] = "",
			"[SmartClass?ClassColor][NameRealm]", -- [1]
			L["Target:"], -- [2]
			L["Guild:"], -- [3]
			L["Guild rank:"], -- [4]
			L["Level:"], -- [5]
			L["Race:"], -- [6]
			L["Class:"], -- [7]
			L["Faction:"], -- [8]
			L["Status:"], -- [9]
			L["Health:"], -- [10]
			"[~HasNoMP?Text([TypePower]:)]", -- [11]
			L["Experience:"], -- [12]
			L["Threat:"], -- [13]
			L["Happiness:"], -- [14]
			L["Zone:"], -- [15]
		},
		rights = {
			['*'] = "",
			"", -- [1]
			"[SmartClass#mouseovertarget?ClassColor#mouseovertarget][IsUnit#player(mouseovertarget)?Text(<<You>>)!NameRealm#mouseovertarget:Text(None)]", -- [2]
			"[Guild:IsEqual([Guild#player])?Green][Guild:Angle]", -- [3]
			"[GuildRank]", -- [4]
			"[Classification] [Level]", -- [5]
			"[SmartRace]", -- [6]
			"[SmartClass:ClassColor]", -- [7]
			"[Faction]", -- [8]
			"[Status]", -- [9]
			"[FractionalHP:Short] [PercentHP:Percent:Paren]", -- [10]
			"[~HasNoMP?Text([FractionalMP:Short] [PercentMP:Percent:Paren])]", -- [11]
			"[CurXP:HideZero?Text([FractionalXP:Short] [PercentXP:Percent:Paren])]", -- [12]
			"[~HasNoThreat?Text([FractionalThreat:Short] [PercentThreat:Percent:Paren])]", -- [13]
			"[IsUnit(pet)?HappyText]", -- [14]
			"[Zone]", -- [15]
		}
	})
end

function CowTip_Text:OnEnable()
	self:RegisterEvent("UPDATE_FACTION")
	
	self:UPDATE_FACTION()
end

local factionList = {}

function CowTip_Text:UPDATE_FACTION()
	for i = 1, GetNumFactions() do
		local name = GetFactionInfo(i)
		factionList[name] = true
	end
end

--[[
How normal tooltips look:
{Name}
{[Guild] : ~~~}
Level {Num} {[(Classification)]}
{NPC? [Faction] : ~~~}
{Offline? Offline : !IsVisible? [Zone] : IsPVP? [PvP] : ~~~}
]]

local lastNumRegistered = 0
local linesToAdd = {}
local linesToAddR = {}
local linesToAddG = {}
local linesToAddB = {}
local linesToAddRight = {}
local linesToAddRightR = {}
local linesToAddRightG = {}
local linesToAddRightB = {}

function CowTip_Text:RefixLines(lastLine)
	for i = 1, lastNumRegistered do
		DogTag:UnregisterFontString(leftGameTooltipStrings[i])
		DogTag:UnregisterFontString(rightGameTooltipStrings[i])
	end
	lastNumRegistered = 0
	
	local unit = CowTip.currentTooltipUnit
	
	for i = lastLine, GameTooltip:NumLines() do
		local left = leftGameTooltipStrings[i]
		local j = i - lastLine + 1
		linesToAdd[j] = left:GetText()
		local r, g, b = left:GetTextColor()
		linesToAddR[j] = r
		linesToAddG[j] = g
		linesToAddB[j] = b
		local right = rightGameTooltipStrings[i]
		if right:IsShown() then
			linesToAddRight[j] = right:GetText()
			local r, g, b = right:GetTextColor()
			linesToAddRightR[j] = r
			linesToAddRightG[j] = g
			linesToAddRightB[j] = b
		end
	end
	
	GameTooltip:ClearLines()
	local db = self.db.profile
	local lefts, rights = db.lefts, db.rights
	local num = 0
	for i = 1, #lefts do
		local left, right = lefts[i], rights[i]
		local left_ev, right_ev
		if left:trim() == "" then
			left = nil
		else
			left_ev = DogTag:Evaluate(unit, left)
			if left_ev:trim() == "" then
				left_ev = nil
			end
		end
		if right:trim() == "" then
			right = nil
		else
			right_ev = DogTag:Evaluate(unit, right)
			if right_ev:trim() == "" then
				right_ev = nil
			end
		end
		if right then
			if right_ev or (left and left_ev and left:find("%[")) then
				num = num + 1
				GameTooltip:AddDoubleLine(left_ev or ' ', right_ev, 1, 1, 1, 1, 1, 1)
				local leftFS = leftGameTooltipStrings[num]
				local rightFS = rightGameTooltipStrings[num]
				DogTag:RegisterFontString(leftFS, GameTooltip, unit, left or ' ')
				DogTag:RegisterFontString(rightFS, GameTooltip, unit, right)
			end
		elseif left then
			if left_ev then
				num = num + 1
				GameTooltip:AddLine(left_ev, 1, 1, 1, true)
				local leftFS = leftGameTooltipStrings[num]
				DogTag:RegisterFontString(leftFS, GameTooltip, unit, left)
			end
		end
	end
	if lastNumRegistered < num then
		lastNumRegistered = num
	end

	for i,left in ipairs(linesToAdd) do
		local right = linesToAddRight[i]
		if right then
			GameTooltip:AddDoubleLine(left, right, linesToAddR[i], linesToAddG[i], linesToAddB[i], linesToAddRightR[i], linesToAddRightG[i], linesToAddRightB[i])
		else
			GameTooltip:AddLine(left, linesToAddR[i], linesToAddG[i], linesToAddB[i], true)
		end
		linesToAdd[i] = nil
		linesToAddR[i] = nil
		linesToAddG[i] = nil
		linesToAddB[i] = nil
		linesToAddRight[i] = nil
		linesToAddRightR[i] = nil
		linesToAddRightG[i] = nil
		linesToAddRightB[i] = nil
	end
end

function CowTip_Text:OnTooltipSetUnit()
	local unit = CowTip.currentTooltipUnit
	
	local num = 2
	if not leftGameTooltipStrings[2]:GetText():find("^" .. LEVEL) then
		-- has a guild
		num = num + 1
	end
	if not UnitPlayerControlled(unit) and not UnitIsPlayer(unit) then
		-- npc, faction line
		local factionText = leftGameTooltipStrings[num+1]:GetText()
		if factionList[factionText] or UnitFactionGroup(unit) then
			num = num + 1
		end
	end
	if not UnitIsConnected(unit) or not UnitIsVisible(unit) or UnitIsPVP(unit) then
		-- offline, zone, or pvp
		num = num + 1
	end
	
	self:RefixLines(num+1)
	
	self:ScheduleRepeatingEvent("CowTip_Text_UpdateTexts", self.UpdateTexts, 1, self)
end

function CowTip_Text:UpdateTexts()
	if not CowTip.currentTooltipUnit then
		return
	end
	self:RefixLines(lastNumRegistered+1)
	
	GameTooltip:Show()
end

function CowTip_Text:OnTooltipHide()
	for i = 1, lastNumRegistered do
		DogTag:UnregisterFontString(leftGameTooltipStrings[i])
		DogTag:UnregisterFontString(rightGameTooltipStrings[i])
	end
	lastNumRegistered = 0
	
	self:CancelScheduledEvent("CowTip_Text_UpdateTexts")
end

local lineConfigBox
function CowTip_Text:CreateLineConfigBox()
	if not lineConfigBox then
		lineConfigBox = CreateFrame("Frame", "CowTipLineConfigFrame", UIParent, "DialogBoxFrame")

		lineConfigBox:SetWidth(750)
		lineConfigBox:SetHeight(650)
		lineConfigBox:SetPoint("CENTER")
		lineConfigBox:SetBackdrop({
			bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]], 
		    edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], 
		    tile = true, tileSize = 16, edgeSize = 16, 
		    insets = { left = 5, right = 5, top = 5, bottom = 5 }
		})
		lineConfigBox:SetBackdropColor(0,0,0,1)
		
		local howto = lineConfigBox:CreateFontString(nil, "ARTWORK", "GameFontHighlightLarge")
		howto:SetPoint("TOP", 0, -16)
		howto:SetText(L["See http://www.wowace.com/wiki/DogTag-1.0 for details on tag usage."])
		
		lineConfigBox:EnableMouse(true)
		lineConfigBox:SetMovable(true)
		lineConfigBox:RegisterForDrag("LeftButton")
		lineConfigBox:SetClampedToScreen(true)
		lineConfigBox:SetScript("OnDragStart", function(this)
			this:StartMoving()
		end)
		lineConfigBox:SetScript("OnDragStop", function(this)
			this:StopMovingOrSizing()
		end)
		
		local function OnEnterPressed(this)
			local num, side = this.num, this.side
			local text = this:GetText() or ''
			
			text = DogTag:FixCasing(text)
			this:SetText(text)
			
			self.db.profile[side == 'left' and 'lefts' or 'rights'][num] = text
		end
		
		local function OnTabPressed(this)
			local num, side = this.num, this.side
			
			this:ClearFocus()
			
			if not IsShiftKeyDown() then
				if side == "left" then
					this:GetParent()["right" .. num]:SetFocus()
				elseif num < 30 then
					this:GetParent()["left" .. (num+1)]:SetFocus()
				end
			else
				if side == "right" then
					this:GetParent()["left" .. num]:SetFocus()
				elseif num > 1 then
					this:GetParent()["right" .. (num-1)]:SetFocus()
				end
			end
		end
		
		for i = 1, 30 do
			for j = 1, 2 do
				local leftSide = j == 1
				local editBox = CreateFrame("EditBox", "CowTipLineConfigFrameEditBox" .. (leftSide and "Left" or "Right") .. i, lineConfigBox)
				local last
				if leftSide then
					lineConfigBox["left" .. i] = editBox
					last = lineConfigBox["left" .. i-1]
				else
					lineConfigBox["right" .. i] = editBox
					last = lineConfigBox["right" .. i-1]
				end
				editBox:SetFontObject(ChatFontNormal)
				editBox:SetHeight(13)
				if last then
					editBox:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -5)
					editBox:SetPoint("TOPRIGHT", last, "BOTTOMRIGHT", 0, -5)
				else
					if leftSide then
						editBox:SetPoint("TOPLEFT", lineConfigBox, "TOPLEFT", 16, -40)
						editBox:SetPoint("TOPRIGHT", lineConfigBox, "TOP", -8, -40)
					else
						editBox:SetPoint("TOPLEFT", lineConfigBox, "TOP", 8, -40)
						editBox:SetPoint("TOPRIGHT", lineConfigBox, "TOPRIGHT", -16, -40)
					end
				end
				editBox:SetJustifyH("LEFT")
		
				local width = editBox:GetWidth()/2 + 10
				local underline = editBox:CreateTexture(nil, "BACKGROUND")
				editBox.underline = underline
				underline:SetTexture("Interface\\Buttons\\WHITE8X8")
				underline:SetHeight(1)
				underline:SetPoint("TOPLEFT", editBox, "BOTTOMLEFT", 0, -1)
				underline:SetPoint("TOPRIGHT", editBox, "BOTTOMRIGHT", 0, -1)
				editBox:SetAutoFocus(false)
				
				editBox:SetScript("OnEscapePressed", editBox.ClearFocus)
				editBox.num = i
				editBox.side = leftSide and "left" or "right"
				
				editBox:SetScript("OnEnterPressed", OnEnterPressed)
				editBox:SetScript("OnEditFocusLost", OnEnterPressed)
				
				editBox:SetScript("OnTabPressed", OnTabPressed)
			end
		end
	end
	lineConfigBox:Show()
	for i = 1, 30 do
		local left = self.db.profile.lefts[i]
		local right = self.db.profile.rights[i]
		lineConfigBox["left" .. i]:SetText(left)
		lineConfigBox["right" .. i]:SetText(right)
	end
end

CowTip_Text:RegisterCowTipOption({
	name = L["Line texts"],
	desc = L["Here you can describe the text on the assorted lines of the tooltip. These use DogTag-1.0 tags. See http://www.wowace.com/wiki/DogTag-1.0 for details."],
	type = 'group',
	args = {
		edit = {
			name = L["Edit"],
			desc = L["Edit the line texts."],
			type = 'execute',
			func = "CreateLineConfigBox",
		}
	}
})
